#version 150
uniform sampler2D sampler0;

// material settings
uniform float materialShininess;
uniform vec3 materialSpecularColor;
uniform vec3 materialEmissiveColor;

//MAX_LIGHTS must be defined, e.g.
//#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec3 intensities; //a.k.a the color of the light
   float constantAttenuation;
   float linearAttenuation;
   float quadraticAttenuation;
   float ambientCoefficient;
   float coneAngle;
   float coneBorder;
   vec3 coneDirection;
} allLights[MAX_LIGHTS];

vec3 applyLight(Light light, vec3 surfaceColor, vec3 surfaceNormal, vec3 surfacePos, vec3 surfaceToCamera) {
    vec3 surfaceToLight = normalize(light.position.xyz * (light.position.w * 2.0 - 1.0) - surfacePos * light.position.w);
    float distanceToLight = length(light.position.xyz - surfacePos);
    float attenuation = 1.0 / (light.constantAttenuation + light.position.w * (light.linearAttenuation * distanceToLight + light.quadraticAttenuation * pow(distanceToLight, 2.0)));
    
    float lightToSurfaceAngle = acos(dot(-surfaceToLight, normalize(light.coneDirection)));
    attenuation -= attenuation * light.position.w * clamp((lightToSurfaceAngle-light.coneAngle)/(light.coneBorder), 0, 1);
    
    //if(light.position.w == 0.0) {
        //directional light
        //surfaceToLight = normalize(-light.position.xyz);
        //attenuation = 1.0; //no attenuation for directional lights
    //} else {
        //point light
        //surfaceToLight = normalize(light.position.xyz - surfacePos);
        //float distanceToLight = length(light.position.xyz - surfacePos);
        //attenuation = 1.0 / (1.0 + light.attenuation * pow(distanceToLight, 2.0));

        //cone restrictions (affects attenuation)
        //float lightToSurfaceAngle = acos(dot(-surfaceToLight, normalize(light.coneDirection)));
        //if(lightToSurfaceAngle > light.coneAngle){
            //attenuation = 0.0;
        //}
    //}

    //ambient
    vec3 ambient = light.ambientCoefficient * surfaceColor.rgb * light.intensities;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(surfaceNormal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.intensities;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, surfaceNormal))), materialShininess);
    vec3 specular = specularCoefficient * materialSpecularColor * light.intensities;

    //linear color (color before gamma correction)
    return ambient + attenuation*(diffuse + specular);
}

in vec4 vertexCoord;
in vec3 normal;
in vec4 color;
in vec4 texcoord0;

out vec4 frag_color;

void main() {
	vec3 surfaceToCamera = normalize(-vertexCoord.xyz);
	vec3 surfaceNormal = normalize(normal);
	
	vec4 surfaceColor = color * texture2DProj(sampler0, texcoord0);
	surfaceColor.rgb *= surfaceColor.rgb;

	vec3 linearColor = vec3(0.0);
	int i = 0;
	for(i = 0; i < numLights; i += 1) {
	    linearColor += applyLight(allLights[i], surfaceColor.rgb, surfaceNormal, vertexCoord.xyz, surfaceToCamera);
	}
	
	//final color (after gamma correction)
	frag_color = vec4(sqrt(linearColor), surfaceColor.a);
	frag_color.rgb += materialEmissiveColor;
}
