#version 150

// material settings
uniform float materialShininess;
uniform vec3 materialSpecularColor;
uniform vec3 materialEmissiveColor;

//MAX_LIGHTS must be defined, e.g.
//#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec3 intensities; //a.k.a the color of the light
   float constantAttenuation;
   float linearAttenuation;
   float quadraticAttenuation;
   float ambientCoefficient;
   float coneAngle;
   float coneBorder;
   vec3 coneDirection;
} allLights[MAX_LIGHTS];

vec3 applyLight(Light light, vec3 surfaceColor, vec3 surfaceNormal, vec3 surfacePos, vec3 surfaceToCamera) {
    vec3 surfaceToLight = normalize(light.position.xyz * (light.position.w * 2.0 - 1.0) - surfacePos * light.position.w);
    float distanceToLight = length(light.position.xyz - surfacePos);
    float attenuation = 1.0 / (light.constantAttenuation + light.position.w * (light.linearAttenuation * distanceToLight + light.quadraticAttenuation * pow(distanceToLight, 2.0)));
    
    float lightToSurfaceAngle = acos(dot(-surfaceToLight, normalize(light.coneDirection)));
    attenuation -= attenuation * light.position.w * clamp((lightToSurfaceAngle-light.coneAngle)/(light.coneBorder), 0, 1);

    //ambient
    vec3 ambient = light.ambientCoefficient * surfaceColor.rgb * light.intensities;

    //diffuse
    float diffuseCoefficient = max(0.0, dot(surfaceNormal, surfaceToLight));
    vec3 diffuse = diffuseCoefficient * surfaceColor.rgb * light.intensities;
    
    //specular
    float specularCoefficient = 0.0;
    if(diffuseCoefficient > 0.0)
        specularCoefficient = pow(max(0.0, dot(surfaceToCamera, reflect(-surfaceToLight, surfaceNormal))), materialShininess);
    vec3 specular = specularCoefficient * materialSpecularColor * light.intensities;

    //linear color (color before gamma correction)
    return ambient + attenuation*(diffuse + specular);
}

// object space to camera space transformation
uniform mat4 modelview_matrix;
 
// camera space to clip coordinates
uniform mat4 projection_matrix;

// texture space to camera space transformation
uniform mat4 texture_matrix;
 
// incoming vertex position
in vec4 vertex_position;
// incoming vertex color
in vec4 vertex_color;
// incoming vertex normal
in vec3 vertex_normal;
// incoming vertex texcoord
in vec4 vertex_texcoord0;
// incoming vertex texcoord
in vec4 vertex_texcoord1;

// eye space vertex coordinate
out vec4 vertexCoord;
// transformed vertex color
out vec4 color;
// eye space vertex normal
out vec3 normal;
// transformed texture coordinate
out vec4 texcoord0;
// transformed texture coordinate
out vec4 texcoord1;

void main() {
	//not a proper transformation if modelview_matrix involves non-uniform scaling(TODO)
	normal = normalize(( modelview_matrix * vec4( vertex_normal, 0 ) ).xyz);
	
	texcoord0 = ( texture_matrix * vertex_texcoord0 );//
	texcoord1 = ( texture_matrix * vertex_texcoord1 );//
	
	// transforming the incoming vertex position
	vertexCoord = modelview_matrix * vertex_position;
	gl_Position = projection_matrix * vertexCoord;
	
	vec3 surfaceToCamera = normalize(-vertexCoord.xyz);
	vec3 surfaceNormal = normalize(normal);
	
	vec4 surfaceColor = vertex_color;
	surfaceColor.rgb *= surfaceColor.rgb;

	vec3 linearColor = vec3(0.0);
	int i = 0;
	for(i = 0; i < numLights; i += 1) {
	    linearColor += applyLight(allLights[i], surfaceColor.rgb, surfaceNormal, vertexCoord.xyz, surfaceToCamera);
	}
	
	//final color (after gamma correction)
	color = vec4(sqrt(linearColor), surfaceColor.a);
	color.rgb += materialEmissiveColor;
}
